module.exports = {
  mixins: [require("vue-mixins/isOpened")],
  filters: {
    notPrevented: require("vue-filters/notPrevented")
  },
  props: {
    "class": {
      type: String,
      "default": ""
    },
    "value": {
      type: String,
      "default": ""
    },
    "transitionIn": {
      type: Function,
      "default": function(arg) {
        var cb, el, style;
        el = arg.el, style = arg.style, cb = arg.cb;
        return cb();
      }
    },
    "transitionOut": {
      type: Function,
      "default": function(arg) {
        var cb, el, style;
        el = arg.el, style = arg.style, cb = arg.cb;
        return cb();
      }
    },
    "maxSize": {
      type: Object,
      "default": function() {
        return {};
      }
    },
    "size": {
      type: Object,
      "default": function() {
        return {};
      }
    },
    "hoverSize": {
      type: Object,
      "default": function() {
        return {};
      }
    }
  },
  data: function() {
    return {
      inner: {},
      content: {},
      current: {},
      lineHeight: 0,
      loaded: false,
      hovered: false,
      style: {
        position: "relative",
        margin: 0,
        padding: 0,
        height: null,
        width: null
      },
      taStyle: {
        height: null,
        width: null,
        overflow: "hidden",
        visibility: "hidden"
      },
      sizerStyle: {
        fontSize: null,
        letterSpacing: null,
        fontFamily: null,
        paddingTop: null,
        paddingRight: null,
        paddingBottom: null,
        paddingLeft: null,
        maxWidth: this.maxSize.width + 'px',
        maxHeight: this.maxSize.height + 'px',
        wordBreak: "break-word",
        whiteSpace: "pre-wrap",
        overflow: "auto"
      }
    };
  },
  watch: {
    maxWidth: function(val) {
      return this.sizerStyle.maxWidth = val + this.inner.top + this.inner.bottom + 'px';
    },
    maxHeight: function(val) {
      return this.sizerStyle.maxHeight = val + this.inner.right + this.inner.left + 'px';
    },
    value: "processContent"
  },
  methods: {
    setHeight: function(height) {
      this.current.height = height;
      this.style.height = height + 'px';
      return this.taStyle.height = this.offsetHeight(height) + 'px';
    },
    setWidth: function(width) {
      this.current.width = width;
      return this.taStyle.width = this.offsetWidth(width) + 'px';
    },
    offsetHeight: function(height) {
      return height - this.inner.top - this.inner.bottom;
    },
    offsetWidth: function(width) {
      return width - this.inner.right - this.inner.left;
    },
    onClick: function(e) {
      if (!(this.opened || this.value !== '')) {
        e.preventDefault();
        this.$els.ta.focus();
      }
      return this.open();
    },
    mouseenter: function() {
      if (!this.opened) {
        if (this.hoverSize.height > this.hoverSize.height || this.hoverSize.width > this.size.width) {
          this.hovered = true;
          return this.move(this.hoverSize);
        }
      }
    },
    mouseleave: function() {
      if (!this.opened && this.hovered) {
        return this.move(this.size);
      }
    },
    process: function() {
      var border, i, len, lh, name, padding, ref, ref1, style, tastyle, upper;
      this.value = (ref = this.$els.content) != null ? ref.textContent : void 0;
      if (!this.size.width) {
        this.size.width = this.$els.ta.offsetWidth;
      }
      if (!this.size.height) {
        this.size.height = this.$els.ta.offsetHeight;
      }
      if (!this.hoverSize.width) {
        this.hoverSize.width = this.size.width;
      }
      if (!this.hoverSize.height) {
        this.hoverSize.height = this.size.height;
      }
      if (this.maxSize.width) {
        if (this.size.width > this.maxSize.width) {
          this.size.width = this.maxSize.width;
        }
      } else {
        this.maxSize.width = this.size.width;
      }
      this.style.width = this.maxSize.width + 'px';
      if (this.maxSize.height) {
        if (this.size.height > this.maxSize.height) {
          this.size.height = this.maxSize.height;
        }
      } else {
        this.maxSize.height = Number.MAX_VALUE;
      }
      style = window.getComputedStyle(this.$el);
      tastyle = window.getComputedStyle(this.$els.ta);
      this.sizerStyle.fontSize = tastyle.getPropertyValue("font-size");
      this.sizerStyle.fontFamily = tastyle.getPropertyValue("font-family");
      this.sizerStyle.letterSpacing = tastyle.getPropertyValue("letter-spacing");
      lh = tastyle.getPropertyValue("line-height");
      if (lh === "normal") {
        this.lineHeight = parseFloat(this.sizerStyle.fontSize.replace("px", ""));
      } else {
        this.lineHeight = parseFloat(lh.replace("px", ""));
      }
      this.sizerStyle.lineHeight = lh;
      ref1 = ["top", "right", "bottom", "left"];
      for (i = 0, len = ref1.length; i < len; i++) {
        name = ref1[i];
        upper = name[0].toUpperCase() + name.slice(1);
        padding = parseFloat(tastyle.getPropertyValue("padding-" + name).replace("px", ""));
        border = parseFloat(tastyle.getPropertyValue("border-" + name + "-width").replace("px", ""));
        this.inner[name] = padding + border;
        this.sizerStyle["padding" + upper] = this.inner[name] + 'px';
      }
      this.processContent();
      if (this.isOpened) {
        this.setHeight(this.content.height);
        this.setWidth(this.content.width);
      } else {
        this.setHeight(this.size.height);
        this.setWidth(this.size.width);
      }
      this.taStyle.visibility = null;
      return this.loaded = true;
    },
    processContent: function(content) {
      this.content.height = this.$els.sizer.offsetHeight + this.lineHeight;
      if (this.content.height < this.size.height) {
        this.content.height = this.size.height;
      }
      if (this.content.height > this.maxSize.height) {
        this.content.height = this.maxSize.height;
      }
      this.content.width = this.$els.sizer.offsetWidth;
      if (this.content.width < this.size.width) {
        this.content.width = this.size.width;
      }
      if (content && this.opened) {
        return this.move(this.content);
      }
    },
    getOffsetStyle: function(style) {
      return {
        height: this.offsetHeight(style.height),
        width: this.offsetWidth(style.width)
      };
    },
    move: function(style, cb) {
      var area;
      area = this.current.height * this.current.width;
      if (area <= style.height * style.width) {
        return this.transitionIn({
          el: this.$els.ta,
          style: this.getOffsetStyle(style),
          cb: (function(_this) {
            return function() {
              _this.setHeight(style.height);
              _this.setWidth(style.width);
              return typeof cb === "function" ? cb() : void 0;
            };
          })(this)
        });
      } else {
        return this.transitionOut({
          el: this.$els.ta,
          style: this.getOffsetStyle(style),
          cb: (function(_this) {
            return function() {
              _this.setHeight(style.height);
              _this.setWidth(style.width);
              return typeof cb === "function" ? cb() : void 0;
            };
          })(this)
        });
      }
    },
    show: function() {
      this.setOpened();
      this.$emit("beforeOpen");
      return this.move(this.content, (function(_this) {
        return function() {
          _this.taStyle.overflow = null;
          return _this.$emit("opened");
        };
      })(this));
    },
    hide: function() {
      this.$emit("beforeClose");
      return this.move(this.size, (function(_this) {
        return function() {
          _this.setClosed();
          _this.taStyle.overflow = "hidden";
          return _this.$emit("closed");
        };
      })(this));
    },
    open: function() {
      if (this.opened) {
        return;
      }
      return this.show();
    },
    close: function() {
      if (!this.opened) {
        return;
      }
      return this.hide();
    },
    toggle: function() {
      if (this.opened) {
        return this.close();
      } else {
        return this.open();
      }
    }
  },
  attached: function() {
    return this.process();
  }
};

if (module.exports.__esModule) module.exports = module.exports.default
;(typeof module.exports === "function"? module.exports.options: module.exports).template = "<div :style=style @mousedown=\"onClick | notPrevented\" @mouseenter=mouseenter @mouseleave=mouseleave><div v-el:sizer=v-el:sizer style=position:absolute;visibility:hidden :style=sizerStyle>{{value}}</div><textarea v-model=value v-el:ta=v-el:ta :style=taStyle :class=class @blur=close></textarea><div style=position:absolute;top:0;left:0;overflow:hidden v-if=\"!isOpened &amp;&amp; value==''\" :style=sizerStyle v-show=loaded><slot name=placeholder></slot></div><div v-el:content=v-el:content style=position:absolute;visibility:hidden v-if=!loaded><slot></slot></div></div>"
